﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool RegisterHotKey(IntPtr hWnd, int id, uint fsModifiers, uint vk);

        public Form1()
        {
            InitializeComponent();
            RegisterHotKey(Handle, 100, 0, (int)Keys.D);
        }

        string _filename;
        private void chooseAnImageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _filename = getFilename();
            if (_filename == null)
                return;
            Bitmap img = MakeBlackAndWhite((Bitmap)Image.FromFile(_filename));
            Size = new Size(img.Width , img.Height);
            using (Graphics g = CreateGraphics())
            {
                g.Clear(Color.CornflowerBlue);
                g.DrawImage(img, 0, 0, Width, Height);               
            }
        }

        public static Bitmap MakeBlackAndWhite(Bitmap original)
        {
            var newBitmap = new Bitmap(original.Width, original.Height);
            Graphics g = Graphics.FromImage(newBitmap);
            var colorMatrix = new ColorMatrix(
           new[]
           {
                 new float[] {1.5f, 1.5f, 1.5f, 0, 0},
                 new float[] {1.5f, 1.5f, 1.5f, 0, 0},
                 new float[] {1.5f, 1.5f, 1.5f, 0, 0},
                 new float[] {0, 0, 0, 1, 0},
                 new float[] {-1, -1, -1, 0, 1}
                 
              });
            var attributes = new ImageAttributes();
            attributes.SetColorMatrix(colorMatrix);
            attributes.SetThreshold(1);
            g.DrawImage(original, new Rectangle(0, 0, original.Width, original.Height),
               0, 0, original.Width, original.Height, GraphicsUnit.Pixel, attributes);
            g.Dispose();
            return newBitmap;
        }

        protected override void WndProc(ref Message m)
        {
            if (m.Msg == 0x0312)
            {
                Draw();
            }
            base.WndProc(ref m);
        }

        private string getFilename()
        {
            using (var o = new OpenFileDialog())
            {
                if (o.ShowDialog() == DialogResult.OK)
                {
                    return o.FileName;
                }
            }
            return null;
        }

        public static void ClickMouse(MouseButtons mouseButton, int x, int y, bool down)
        {
            var inputs = new User32.INPUT[1];

            var input = new User32.INPUT { type = 0 };

            input.U.mi.dx = x;
            input.U.mi.dy = y;

            switch (mouseButton)
            {
                case MouseButtons.Left:
                    input.U.mi.dwFlags = down ? User32.MOUSEEVENTF.LEFTDOWN : User32.MOUSEEVENTF.LEFTUP;
                    break;
                case MouseButtons.Right:
                    input.U.mi.dwFlags = down ? User32.MOUSEEVENTF.RIGHTDOWN : User32.MOUSEEVENTF.RIGHTUP;
                    break;
                case MouseButtons.Middle:
                    input.U.mi.dwFlags = down ? User32.MOUSEEVENTF.MIDDLEDOWN : User32.MOUSEEVENTF.MIDDLEUP;
                    break;
                case MouseButtons.XButton1:
                case MouseButtons.XButton2:
                    input.U.mi.mouseData = (mouseButton == MouseButtons.XButton1) ? 0x0001 : 0x0002;
                    input.U.mi.dwFlags = down ? User32.MOUSEEVENTF.XDOWN : User32.MOUSEEVENTF.XUP;
                    break;
            }

            inputs[0] = input;

            User32.SendInput(1, inputs, User32.INPUT.Size);
        }


        void Draw()
        {
            Bitmap img = MakeBlackAndWhite((Bitmap)Image.FromFile(_filename));

            Point startPos = Cursor.Position;
            ClickMouse(MouseButtons.Left, startPos.X, startPos.Y, true);
            ClickMouse(MouseButtons.Left, startPos.X, startPos.Y, false);
            for (int y = 0; y < img.Height; y++)
            {
                for (int x = 0; x < img.Width; x++)
                {
                    Color c = img.GetPixel(x, y);

                    if (c.B == 0)
                    {
                        int drawX = startPos.X + x;
                        int drawY = startPos.Y + y;
                        MoveMouse(drawX, drawY);
                        ClickMouse(MouseButtons.Left, drawX, drawY, true);
                        ClickMouse(MouseButtons.Left, drawX, drawY, false);
                        Thread.Sleep(1);
                    }
                }
            }
            MessageBox.Show("Fertig!");
        }

        public static void MoveMouse(int x, int y)
        {
            User32.SetCursorPos(x, y);
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            {
                _filename = getFilename();
                if (_filename == null)
                    return;
                Bitmap img = MakeBlackAndWhite((Bitmap)Image.FromFile(_filename));
                BackgroundImage = Image.FromFile(_filename);
                Size = new Size(img.Width + 100, img.Height + 100);
                using (Graphics g = CreateGraphics())
                {
                    g.Clear(Color.CornflowerBlue);
                    g.DrawImage(img, 0, 0, Width, Height);
                }
            }
        }
    }
}